@@ -24,6 +24,9 @@
 
 package Init::Services;
 
+my $SERVICE_START = 0;
+my $SERVICE_STOP  = 1;
+
 use Init::ServicesList;
 
 sub get_runlevels
@@ -48,6 +51,7 @@ sub get_runlevels
      "slackware-9.1.0"  => "slackware-9.1.0",
      "gentoo"           => "gentoo",
      "freebsd-5"        => "slackware-9.1.0",
+     "solaris-2.11"     => "freebsd-5",
     );
 
   %runlevels=
@@ -77,7 +81,7 @@ sub get_default_runlevel
 {
   my $type = &get_init_type ();
 
-  return "default" if ($type eq "gentoo" || $type eq "rcng" || $type eq "bsd");
+  return "default" if ($type eq "gentoo" || $type eq "rcng" || $type eq "bsd" || $type eq "smf");
   return &get_sysv_default_runlevel ();
 }
 
@@ -100,6 +104,7 @@ sub get_sysv_paths
      "pld-1.0"        => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "vine-3.0"       => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "ark"            => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
+     "solaris-2.11"   => ["$gst_prefix/etc",      "$gst_prefix/etc/init.d",      "../init.d"],
      );
   my $res;
 
@@ -135,11 +140,11 @@ sub get_sysv_runlevels_status
 
     if ($action eq "S")
 		{
-      push @arr, [ $runlevel, "start", $priority ];
+      push @arr, [ $runlevel, $SERVICE_START, $priority ];
     }
 		elsif ($action eq "K")
 		{
-      push @arr, [ $runlevel, "stop", $priority ];
+      push @arr, [ $runlevel, $SERVICE_STOP, $priority ];
 		}
 	}
 	
@@ -251,7 +256,7 @@ sub set_sysv_service
   foreach $r (@$runlevels)
   {
     $runlevel = $$r[0];
-    $action   = ($$r[1] eq "start") ? "S" : "K";
+    $action   = ($$r[1] == $SERVICE_START) ? "S" : "K";
     $priority = sprintf ("%0.2d", $$r[2]);
 
     $configured_runlevels{$runlevel} = 1;
@@ -263,7 +268,7 @@ sub set_sysv_service
 
       if ($runlevel eq $default_runlevel)
       {
-        &run_sysv_initd_script ($script, $$r[1]);
+        &run_sysv_initd_script ($script, ($$r[1] == $SERVICE_START) ? "start" : "stop");
       }
     }
   }
@@ -310,7 +315,7 @@ sub get_filerc_runlevels_status
 
     foreach $runlevel (@runlevels)
     {
-      push @arr, [ $runlevel, "start", $priority ];
+      push @arr, [ $runlevel, $SERVICE_START, $priority ];
     }
   }
 
@@ -322,7 +327,7 @@ sub get_filerc_runlevels_status
 
     foreach $runlevel (@runlevels)
     {
-      push @arr, [ $runlevel, "stop", $priority ];
+      push @arr, [ $runlevel, $SERVICE_STOP, $priority ];
     }
   }
 
@@ -414,28 +419,28 @@ sub set_filerc_service
     $priority = 50 if ($priority == 0); #very rough guess
     $configured_runlevels {$$i[0]} = 1;
 
-    if ($$i[1] eq "start")
+    if ($$i[1] == $SERVICE_START)
     {
-      $hash{$priority}{"start"} = [] if (!$hash{$priority}{"start"});
-      push @{$hash{$priority}{"start"}}, $$i[0];
+      $hash{$priority}{$SERVICE_START} = [] if (!$hash{$priority}{$SERVICE_START});
+      push @{$hash{$priority}{$SERVICE_START}}, $$i[0];
     }
     else
     {
-      $hash{$priority}{"stop"} = [] if (!$hash{$priority}{"stop"});
-      push @{$hash{$priority}{"stop"}}, $$i[0];
+      $hash{$priority}{$SERVICE_STOP} = [] if (!$hash{$priority}{$SERVICE_STOP});
+      push @{$hash{$priority}{$SERVICE_STOP}}, $$i[0];
     }
 
     if ($$i[0] eq $default_runlevel)
     {
-      &run_sysv_initd_script ($script, $$i[1]);
+      &run_sysv_initd_script ($script, ($$i[1] == $SERVICE_START) ? "start" : "stop");
     }
   }
 
   foreach $priority (keys %hash)
   {
     $line  = sprintf ("%0.2d", $priority) . "\t";
-    $line .= &concat_filerc_runlevels (@{$hash{$priority}{"stop"}}) . "\t";
-    $line .= &concat_filerc_runlevels (@{$hash{$priority}{"start"}}) . "\t";
+    $line .= &concat_filerc_runlevels (@{$hash{$priority}{$SERVICE_STOP}}) . "\t";
+    $line .= &concat_filerc_runlevels (@{$hash{$priority}{$SERVICE_START}}) . "\t";
     $line .= $initd_path . "/" . $script . "\n";
 
     push @$buff, $line;
@@ -494,45 +499,10 @@ sub set_filerc_services
 }
 
 # this functions get a list of the services that run on a bsd init
-sub get_bsd_service_info
+sub get_bsd_scripts_list
 {
-  my ($service) = @_;
-  my ($script);
-  my (%hash);
-	my (@arr, @rl);
-
-  $script = $service;
-  $script =~ s/^.*\///;
-  $script =~ s/^rc\.//;
-
-  return undef if (! Utils::File::exists ($service));
-
-  $hash {"script"} = $service;
-
-  # we hardcode the fourth runlevel, it's the graphical one
-  if ( -x $service)
-  {
-    push @arr, { "name"   => 4,
-                 "action" => "start" };
-  }
-  else
-  {
-    push @arr, { "name"   => 4,
-                 "action" => "stop" };
-  }
-
-	push @rl, { "runlevel" => \@arr };
-  
-	$hash{"runlevels"} = \@rl;
-  
-  return \%hash;
-}
-
-sub get_bsd_services
-{
-  my (%ret);
   my ($files) = [ "rc.M", "rc.inet2", "rc.4" ];
-  my ($file);
+  my ($file, $i, @scripts);
 
   foreach $i (@$files)
   {
@@ -550,22 +520,59 @@ sub get_bsd_services
 
       if ($line =~ /^if[ \t]+\[[ \t]+\-x[ \t]([0-9a-zA-Z\/\.\-_]+) .*\]/)
       {
-        my (%hash);
         $service = $1;
 
-        $hash = &get_bsd_service_info ($service);
-
-        if ($hash ne undef)
-        {
-          $ret{$service} = $hash;
-        }
+        push @scripts, $service;
       }
     }
 
-    Utils::File::close_file ($fd);
+    &Utils::File::close_file ($fd);
   }
 
-  return \%ret;
+  return \@scripts;
+}
+
+sub get_bsd_service_info
+{
+  my ($service) = @_;
+  my ($script, @runlevels);
+
+  $script = $service;
+  $script =~ s/^.*\///;
+  $script =~ s/^rc\.//;
+
+  return undef if (! Utils::File::exists ($service));
+
+  # hardcode the fourth runlevel, it's the graphical one
+  if ( -x $service)
+  {
+    push @runlevels, [ "4", $SERVICE_START, 0 ];
+  }
+  else
+  {
+    push @runlevels, [ "4", $SERVICE_STOP, 0 ];
+  }
+
+  # FIXME: $service contains full path, would
+  # be nice to just show the $script name
+  return ($service, $runlevels);
+}
+
+sub get_bsd_services
+{
+  my (@arr, @scripts, $script);
+
+  $scripts = &get_bsd_scripts_list ();
+
+  foreach $script (@scripts)
+  {
+    my (@info);
+
+    @info = &get_bsd_service_info ($script);
+    push @arr, \@info if (scalar (@info));
+  }
+
+  return \@arr;
 }
 
 sub run_bsd_script
@@ -592,20 +599,41 @@ sub run_bsd_script
 }
 
 # This function stores the configuration in a bsd init
+sub get_bsd_service_path
+{
+  my ($script) = @_;
+  my (@scripts, $elem, $i);
+
+  $scripts = &get_bsd_scripts_list ();
+
+  foreach $i (@scripts)
+  {
+    $elem = $i;
+    $elem =~ s/^.*\///;
+    $elem =~ s/^rc\.//;
+
+    return $i if ($elem eq $script);
+  }
+
+  return undef;
+}
+
 sub set_bsd_services
 {
   my ($services) = @_;
-  my ($script, $runlevels);
+  my ($script, $runlevels, @scripts);
 
-	foreach $service (@$services)
-	{
-    $script = $$service[0];
+  foreach $service (@$services)
+  {
+    $script = &get_bsd_service_path ($$service[0]);
     $runlevels = $$service[1];
     $runlevel  = $$runlevels[0];
 
+    next if ($script eq undef);
+
     $action = $$runlevel[1];
 
-    if ($action eq "start")
+    if ($action == $SERVICE_START)
     {
       &Utils::File::run ("chmod ugo+x $script");
       &run_bsd_script ($script, "start");
@@ -734,11 +762,11 @@ sub get_gentoo_runlevel_status_by_service
   {
     if (defined $start_runlevels{$runlevel})
     {
-      push @arr, [ $runlevel, "start", 0 ];
+      push @arr, [ $runlevel, $SERVICE_START, 0 ];
     }
     else
     {
-      push @arr, [ $runlevel, "stop", 0 ];
+      push @arr, [ $runlevel, $SERVICE_STOP, 0 ];
     }
   }
 
@@ -799,7 +827,7 @@ sub set_gentoo_service_status
 {
   my ($script, $rl, $action) = @_;
 
-  if ($action eq "start")
+  if ($action == $SERVICE_START)
   {
     &Utils::File::run ("rc-update add $script $rl");
     &run_gentoo_script ($script, "start");
@@ -857,53 +885,43 @@ sub get_rcng_status_by_service
       }
     }
 
-    Utils::File::close_file ($fd);
+    &Utils::File::close_file ($fd);
     return $active;
   }
 }
 
 sub get_rcng_service_info
 {
-  my ($service) = @_;
-  my ($script, @actions, @runlevels);
-  my (%hash, @arr, @rl);
+  my ($script) = @_;
+  my (@runlevels);
 
-  $hash{"script"} = $service;
-
-  if (get_rcng_status_by_service ($service))
+  if (get_rcng_status_by_service ($script))
   {
-    push @arr, { "name"   => "default",
-                 "action" => "start" };
+    push @runlevels, [ "default", $SERVICE_START, 0 ];
   }
   else
   {
-    push @arr, { "name"   => "default",
-                 "action" => "stop" };
+    push @runlevels, [ "default", $SERVICE_STOP, 0 ];
   }
 
-  push @rl,  { "runlevel", \@arr };
-
-  $hash {"runlevels"} = \@rl;
-
-  return \%hash;
+  return ($script, $runlevels);
 }
 
 sub get_rcng_services
 {
   my ($service);
-  my (%ret);
+  my (@arr);
 
   foreach $service (<$gst_prefix/etc/rc.d/*>)
   {
-    my (%hash);
+    my (@info);
 
     $service =~ s/.*\///;
-    $hash = &get_rcng_service_info ($service);
-
-    $ret{$service} = $hash if ($hash ne undef);
+    @info = &get_rcng_service_info ($service);
+    push @arr, \@info if (scalar (@info));
   }
 
-  return \%ret;
+  return \@arr;
 }
 
 sub run_rcng_script
@@ -1017,7 +1035,7 @@ sub set_rcng_services
     $script    = $$service[0];
     $runlevels = $$service[1];
     $runlevel  = $$runlevels[0];
-    $action    = ($$runlevel[1] eq "start")? 1 : 0;
+    $action    = ($$runlevel[1] == $SERVICE_START)? 1 : 0;
 
     &$func ($script, $action);
   }
@@ -1027,54 +1045,43 @@ sub set_rcng_services
 sub get_suse_service_info ($service)
 {
   my ($service) = @_;
-  my (%hash, @arr, @ret);
+  my (@runlevels, $link, $runlevel);
                                                                                                                                                              
-  $hash{"script"} = $service;
-
   foreach $link (<$rcd_path/rc[0-9S].d/S[0-9][0-9]$service>)
   {
     $link =~ s/$rcd_path\///;
     $link =~ /rc([0-6])\.d\/S[0-9][0-9].*/;
     $runlevel = $1;
 
-    push @arr, { "name"   => $runlevel,
-                 "action" => "start" };
+    push @runlevels, [ $runlevel, $SERVICE_START, 0 ];
   }
 
   foreach $link (<$rcd_path/boot.d/S[0-9][0-9]$service>)
   {
-    push @arr, {"name"   => "B",
-                "action" => "start" };
+    push @runlevels, [ "B", $SERVICE_START, 0 ];
   }
 
-  if (scalar @arr > 0)
-  {
-    push @ret, { "runlevel" => \@arr };
-    $hash{"runlevels"} = \@ret;
-  }
-
-  return \%hash;
+  return ($service, $runlevels);
 }
 
 sub get_suse_services
 {
-  my ($service, %ret);
+  my ($service, @arr);
 
   ($rcd_path, $initd_path) = &get_sysv_paths ();
 
   foreach $service (<$gst_prefix/etc/init.d/*>)
   {
-    my (%hash);
+    my (@info);
 
     next if (-d $service || ! -x $service);
 
     $service =~ s/.*\///;
-    $hash = &get_suse_service_info ($service);
-
-    $ret{$service} = $hash if ($hash ne undef);
+    @info = &get_suse_service_info ($service);
+    push @arr, \@info  if (scalar (@info));
   }
 
-  return \%ret;
+  return \@arr;
 }
 
 # This function stores the configuration in suse init
@@ -1099,12 +1106,12 @@ sub set_suse_services
     {
       $configured_runlevels{$$rl[0]} = 1;
 
-      if ($$rl[1] eq "start")
+      if ($$rl[1] == $SERVICE_START)
       {
         $rllist .= $$rl[0] . ",";
       }
 
-      &run_sysv_initd_script ($script, $$rl[1]);
+      &run_sysv_initd_script ($script, ($$rl[1] == $SERVICE_START) ? "start" : "stop");
     }
 
     if ($rllist ne "")
@@ -1121,6 +1128,158 @@ sub set_suse_services
   }
 }
 
+# functions to get/set services info in smf
+sub smf_service_exists
+{
+  my($service) = @_;
+  my($services) = &get_smf_services_list ();
+
+  foreach $i (@$services)
+  {
+    return 1 if ($i =~ /$service/);
+  }
+
+  return 0;
+}
+
+sub run_smf_svcadm
+{
+  my ($service, $arg) = @_;
+  my ($option);
+
+  my %op =
+    ("stop" => "disable",
+     "start" => "enable"
+    );
+
+  if (&smf_service_exists ($service))
+  {
+    if (!&Utils::File::run ("svcadm $op{$arg} $service"))
+    {
+      &Utils::Report::do_report ("service_sysv_op_success", $service, $arg);
+      &Utils::Report::leave ();
+      return 0;
+    }
+  }
+
+  &Utils::Report::do_report ("service_sysv_op_failed", $service, $arg);
+  &Utils::Report::leave ();
+  return -1;
+}
+
+sub get_smf_runlevel_status_by_service
+{
+  my ($service, $status) = @_;
+  my (@arr);
+
+  if ($status)
+  {
+    push @arr, [ "default", $SERVICE_START, 0 ];
+  }
+  else
+  {
+    push @arr, [ "default", $SERVICE_STOP, 0 ];
+  }
+
+  return \@arr;
+}
+
+sub get_smf_service_info
+{
+  my ($service) = @_;
+  my ($fd, @runlevels);
+  my $status = 0;
+
+  $fd = &Utils::File::run_pipe_read ("svcs -l $service");
+
+  while (<$fd>)
+  {
+    $status = 1 if (/^state.*online/);
+  }
+
+  &Utils::File::close_file ($fd);
+
+  $runlevels = &get_smf_runlevel_status_by_service ($service, $status);
+  $service =~ m/.*\/(.*)$/;
+
+  return ($service, $runlevels);
+}
+
+sub get_smf_services_list
+{
+  my ($fd, @list);
+
+  $fd = &Utils::File::run_pipe_read ("svcs -H -a");
+
+  while (<$fd>)
+  {
+    next if (/svc:\/milestone/);
+
+    if (/^.*\s*.*\s*svc:\/(.*):.*/)
+    {
+      push @list, $1;
+    }
+  }
+
+  &Utils::File::close_file ($fd);
+  return \@list;
+}
+
+sub get_smf_services
+{
+  my ($service, @arr);
+  my ($service_list) = &get_smf_services_list ();
+
+  foreach $service (@$service_list)
+  {
+    my (@info);
+
+    @info = &get_smf_service_info ($service);
+    push @arr, \@info if scalar (@info);
+  }
+
+  return \@arr;
+}
+
+sub set_smf_service_status
+{
+  my ($service, $rl, $active) = @_;
+  my ($info);
+
+  $info = &get_smf_service_info ($service);
+
+  #return if service has not changed
+  return if ($active == @{@$info[0]}[1]);
+
+  if ($active == $SERVICE_START)
+  {
+    &Utils::File::run ("svcadm enable -s $service");
+  }
+  else
+  {
+    &Utils::File::run ("svcadm disable -s $service");
+  }
+}
+
+sub set_smf_services
+{
+  my ($services) = @_;
+  my ($action, $rl, $script, $arr);
+
+  foreach $service (@$services)
+  {
+    $script = $$service[0];
+    $arr = $$service[1];
+
+    foreach $i (@$arr)
+    {
+      $action = $$i[1];
+      $rl = $$i[0];
+      &set_smf_service_status ($script, $rl, $action);
+    }
+  }
+}
+
 # generic functions to get the available services
 sub get_init_type
 {
@@ -1148,6 +1307,10 @@ sub get_init_type
   {
     return "suse";
   }
+  elsif ($gst_dist =~ /solaris/)
+  {
+    return "smf";
+  }
   else
   {
     return "sysv";
@@ -1166,6 +1329,7 @@ sub run_script
      "gentoo"  => \&run_gentoo_script,
      "rcng"    => \&run_rcng_script,
      "suse"    => \&run_sysv_initd_script,
+     "smf"     => \&run_smf_svcadm,
     );
 
   $type = &get_init_type ();
@@ -1183,6 +1347,7 @@ sub get
   return &get_gentoo_services () if ($type eq "gentoo");
   return &get_rcng_services ()   if ($type eq "rcng");
   return &get_suse_services ()   if ($type eq "suse");
+  return &get_smf_services ()    if ($type eq "smf");
 
   return undef;
 }
@@ -1199,6 +1364,7 @@ sub set
   &set_gentoo_services ($services) if ($type eq "gentoo");
   &set_rcng_services   ($services) if ($type eq "rcng");
   &set_suse_services   ($services) if ($type eq "suse");
+  &set_smf_services    ($services) if ($type eq "smf");
 }
 
 1;
